@@ -54,9 +54,9 @@ impl<'a, 'app> CardDecryptor<'a, 'app> {
                 )))
             }
         } else {
-            Err(Error::InternalError(format!(
-                "Failed to get the decryption key's Fingerprint from the card"
-            )))
+            Err(Error::InternalError(
+                "Failed to get the decryption key's Fingerprint from the card".to_string(),
+            ))
         }
     }
 }
